from math import floor


def round_down(x: float, sig_digits: int) -> float:
    return floor(x * (10**sig_digits)) / (10**sig_digits)


def to_token_decimals(x: float) -> int:
    return int((10**6) * x)



from unittest import TestCase
from py_clob_client.constants import MUMBAI

from py_clob_client.signer import Signer
from py_clob_client.signing.eip712 import sign_clob_auth_message

# publicly known private key
private_key = "0xac0974bec39a17e36ba4a6b4d238ff944bacb478cbed5efcae784d7bf4f2ff80"
chain_id = MUMBAI
signer = Signer(private_key=private_key, chain_id=chain_id)


class TestEIP712(TestCase):
    def test_sign_clob_auth_message(self):
        signature = sign_clob_auth_message(signer, 10000000, 23)
        self.assertIsNotNone(signature)
        self.assertEqual(
            signature,
            "0xd91760ebcb14e814e9e12600b9bc7cd6bf13ebc175f6a28538b4925763f94f90012da34dd71290d441c28bc4f9b2281d3eb9ecfd1c9a63db1ce9ca85c89c914c1b",
        )



from unittest import TestCase

from py_clob_client.clob_types import OrderArgs
from py_clob_client.constants import MUMBAI
from py_clob_client.order_builder.constants import BUY, SELL

from py_clob_client.signer import Signer
from py_clob_client.order_builder.builder import OrderBuilder
from py_order_utils.model import POLY_GNOSIS_SAFE, EOA

# publicly known private key
private_key = "0xac0974bec39a17e36ba4a6b4d238ff944bacb478cbed5efcae784d7bf4f2ff80"
chain_id = MUMBAI
signer = Signer(private_key=private_key, chain_id=chain_id)


class TestOrderBuilder(TestCase):
    def test_create_order_buy(self):
        builder = OrderBuilder(signer)

        signed_order = builder.create_order(
            order_args=OrderArgs(
                token_id="123",
                price=0.56,
                size=21.04,
                side=BUY,
                fee_rate_bps=111,
                nonce=123,
                expiration=50000,
            )
        )

        self.assertTrue(isinstance(signed_order.order["salt"], int))
        self.assertIsNotNone(signed_order)
        self.assertEqual(
            signed_order.order["maker"],
            "0xf39Fd6e51aad88F6F4ce6aB8827279cffFb92266",
        )
        self.assertEqual(
            signed_order.order["signer"],
            "0xf39Fd6e51aad88F6F4ce6aB8827279cffFb92266",
        )
        self.assertEqual(
            signed_order.order["taker"],
            "0x0000000000000000000000000000000000000000",
        )
        self.assertEqual(
            signed_order.order["tokenId"],
            123,
        )
        self.assertEqual(
            signed_order.order["makerAmount"],
            11782400,
        )
        self.assertEqual(
            signed_order.order["takerAmount"],
            21040000,
        )
        self.assertEqual(
            signed_order.order["side"],
            0,
        )
        self.assertEqual(
            signed_order.order["expiration"],
            50000,
        )
        self.assertEqual(
            signed_order.order["nonce"],
            123,
        )
        self.assertEqual(
            signed_order.order["feeRateBps"],
            111,
        )
        self.assertEqual(
            signed_order.order["signatureType"],
            EOA,
        )
        self.assertIsNotNone(signed_order.signature)
        self.assertEqual(
            int(signed_order.order["makerAmount"])
            / int(signed_order.order["takerAmount"]),
            0.56,
        )

    def test_create_order_sell(self):
        builder = OrderBuilder(signer, sig_type=POLY_GNOSIS_SAFE)

        signed_order = builder.create_order(
            order_args=OrderArgs(
                token_id="123",
                price=0.56,
                size=21.04,
                side=SELL,
                fee_rate_bps=111,
                nonce=123,
                expiration=50000,
            )
        )

        self.assertTrue(isinstance(signed_order.order["salt"], int))
        self.assertIsNotNone(signed_order)
        self.assertEqual(
            signed_order.order["maker"],
            "0xf39Fd6e51aad88F6F4ce6aB8827279cffFb92266",
        )
        self.assertEqual(
            signed_order.order["signer"],
            "0xf39Fd6e51aad88F6F4ce6aB8827279cffFb92266",
        )
        self.assertEqual(
            signed_order.order["taker"],
            "0x0000000000000000000000000000000000000000",
        )
        self.assertEqual(
            signed_order.order["tokenId"],
            123,
        )
        self.assertEqual(
            signed_order.order["makerAmount"],
            21040000,
        )
        self.assertEqual(
            signed_order.order["takerAmount"],
            11782400,
        )
        self.assertEqual(
            signed_order.order["side"],
            1,
        )
        self.assertEqual(
            signed_order.order["expiration"],
            50000,
        )
        self.assertEqual(
            signed_order.order["nonce"],
            123,
        )
        self.assertEqual(
            signed_order.order["feeRateBps"],
            111,
        )
        self.assertEqual(
            signed_order.order["signatureType"],
            POLY_GNOSIS_SAFE,
        )
        self.assertIsNotNone(signed_order.signature)
        self.assertEqual(
            int(signed_order.order["takerAmount"])
            / int(signed_order.order["makerAmount"]),
            0.56,
        )



from unittest import TestCase
from py_clob_client.clob_types import FilterParams

from py_clob_client.http_helpers.helpers import build_query_params, add_query_params


class TestHelpers(TestCase):
    def test_build_query_params(self):
        # last is ?
        url = build_query_params("http://tracker?", "q1", "a")
        self.assertIsNotNone(url)
        self.assertEqual(url, "http://tracker?q1=a")

        # last is not ?
        url = build_query_params("http://tracker?q1=a", "q2", "b")
        self.assertIsNotNone(url)
        self.assertEqual(url, "http://tracker?q1=a&q2=b")

    def test_add_query_params(self):
        url = add_query_params(
            "http://tracker",
            FilterParams(market="10000", max=250, start_ts=1450000, end_ts=1460000),
        )
        self.assertIsNotNone(url)
        self.assertEqual(
            url, "http://tracker?market=10000&max=250&startTs=1450000&endTs=1460000"
        )



import requests

from py_clob_client.clob_types import FilterParams

from ..exceptions import PolyApiException

GET = "GET"
POST = "POST"
DELETE = "DELETE"
PUT = "PUT"


def request(endpoint: str, method: str, headers=None, data=None):
    try:
        resp = requests.request(
            method=method, url=endpoint, headers=headers, json=data if data else None
        )
        if resp.status_code != 200:
            raise PolyApiException(resp)
        return resp.json()
    except requests.RequestException:
        raise PolyApiException(error_msg="Request exception!")


def post(endpoint, headers=None, data=None):
    return request(endpoint, POST, headers, data)


def get(endpoint, headers=None, data=None):
    return request(endpoint, GET, headers, data)


def delete(endpoint, headers=None, data=None):
    return request(endpoint, DELETE, headers, data)


def build_query_params(url: str, param: str, val: str) -> str:
    url_with_params = url
    last = url_with_params[-1]
    # if last character in url string == "?", append the param directly: api.com?param=value
    if last == "?":
        url_with_params = "{}{}={}".format(url_with_params, param, val)
    else:
        # else add "&", then append the param
        url_with_params = "{}&{}={}".format(url_with_params, param, val)
    return url_with_params


def add_query_params(base_url: str, params: FilterParams = None) -> str:
    """
    Adds query parameters to a url
    """
    url = base_url
    if params:
        url = url + "?"
        if params.market:
            url = build_query_params(url, "market", params.market)
        if params.max:
            url = build_query_params(url, "max", params.max)
        if params.start_ts:
            url = build_query_params(url, "startTs", params.start_ts)
        if params.end_ts:
            url = build_query_params(url, "endTs", params.end_ts)
    return url



from requests import Response


class PolyException(Exception):
    def __init__(self, msg):
        self.msg = msg


class PolyApiException(PolyException):
    def __init__(self, resp: Response = None, error_msg=None):
        assert resp is not None or error_msg is not None
        if resp is not None:
            self.status_code = resp.status_code
            self.error_msg = self._get_message(resp)
        if error_msg is not None:
            self.error_msg = error_msg
            self.status_code = None

    def _get_message(self, resp: Response):
        try:
            return resp.json()
        except Exception:
            return resp.text

    def __repr__(self):
        return "PolyApiException[status_code={}, error_message={}]".format(
            self.status_code, self.error_msg
        )

    def __str__(self):
        return self.__repr__()



from py_order_utils.builders import OrderBuilder as UtilsOrderBuild
from py_order_utils.model import EOA, OrderData, SignedOrder
from py_order_utils.config import get_contract_config

from .helpers import to_token_decimals
from .constants import BUY

from ..signer import Signer
from ..clob_types import OrderArgs


class OrderBuilder:
    def __init__(self, signer: Signer, sig_type=None, funder=None):
        self.signer = signer

        # Signature type used sign orders, defaults to EOA type
        self.sig_type = sig_type if sig_type is not None else EOA

        # Address which holds funds to be used.
        # Used for Polymarket proxy wallets and other smart contract wallets
        # Defaults to the address of the signer
        self.funder = funder if funder is not None else self.signer.address()
        self.contract_config = self._get_contract_config(self.signer.get_chain_id())
        self.order_builder = UtilsOrderBuild(
            self.contract_config.exchange, self.signer.get_chain_id(), self.signer
        )

    def _get_contract_config(self, chain_id: int):
        return get_contract_config(chain_id)

    def create_order(self, order_args: OrderArgs) -> SignedOrder:
        """
        Creates and signs an order
        """
        if order_args.side == BUY:
            side = 0
            maker_amount = to_token_decimals(order_args.price * order_args.size)
            taker_amount = to_token_decimals(order_args.size)
        else:
            side = 1
            maker_amount = to_token_decimals(order_args.size)
            taker_amount = to_token_decimals(order_args.price * order_args.size)

        data = OrderData(
            maker=self.funder,
            taker=order_args.taker,
            tokenId=order_args.token_id,
            makerAmount=maker_amount,
            takerAmount=taker_amount,
            side=side,
            feeRateBps=str(order_args.fee_rate_bps),
            nonce=str(order_args.nonce),
            signer=self.signer.address(),
            expiration=str(order_args.expiration),
            signatureType=self.sig_type,
        )

        return self.order_builder.build_signed_order(data)



TIME = "/time"
CREATE_API_KEY = "/create-api-key"
GET_API_KEYS = "/get-api-keys"
DELETE_API_KEY = "/delete-api-key"
DERIVE_API_KEY = "/derive-api-key"
TRADE_HISTORY = "/trade-history"
GET_ORDER_BOOK = "/book"
GET_ORDER = "/order/"
OPEN_ORDERS = "/open-orders"
LARGE_ORDERS = "/large-orders"
POST_ORDER = "/order"
CANCEL = "/order"
ORDER_HISTORY = "/order-history"
CANCEL_ALL = "/cancel-all"
MID_POINT = "/midpoint"
PRICE = "/price"
GET_LAST_TRADE_PRICE = "/last-trade-price"



from unittest import TestCase

from py_clob_client.signing.hmac import build_hmac_signature


class TestHMAC(TestCase):
    def test_build_hmac_signature(self):
        signature = build_hmac_signature(
            "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA=",
            "1000000",
            "test-sign",
            "/orders",
            '{"hash": "0x123"}',
        )
        self.assertIsNotNone(signature)
        self.assertEqual(
            signature,
            "ZwAdJKvoYRlEKDkNMwd5BuwNNtg93kNaR_oU2HrfVvc=",
        )



from dataclasses import dataclass
from typing import Any

from .constants import ZERO_ADDRESS


@dataclass
class ApiCreds:
    api_key: str
    api_secret: str
    api_passphrase: str


@dataclass
class RequestArgs:
    method: str
    request_path: str
    body: Any = None


@dataclass
class OrderArgs:
    token_id: str
    """
    TokenID of the Conditional token asset being traded
    """

    price: float
    """
    Price used to create the order
    """

    size: float
    """
    Size in terms of the ConditionalToken
    """

    side: str
    """
    Side of the order
    """

    fee_rate_bps: int = 0
    """
    Fee rate, in basis points, charged to the order maker, charged on proceeds
    """

    nonce: int = 0
    """
    Nonce used for onchain cancellations
    """

    expiration: int = 0
    """
    Timestamp after which the order is expired.
    """

    taker: str = ZERO_ADDRESS
    """
    Address of the order taker. The zero address is used to indicate a public order
    """


@dataclass
class FilterParams:
    market: str = None
    max: int = None
    start_ts: int = None
    end_ts: int = None
    min_value: str = None



from datetime import datetime
from unittest import TestCase
from py_clob_client.clob_types import ApiCreds, RequestArgs
from py_clob_client.constants import MUMBAI
from py_clob_client.headers.headers import (
    POLY_ADDRESS,
    POLY_API_KEY,
    POLY_NONCE,
    POLY_PASSPHRASE,
    POLY_SIGNATURE,
    POLY_TIMESTAMP,
    create_level_1_headers,
    create_level_2_headers,
)
from py_clob_client.signer import Signer

# publicly known private key
private_key = "0xac0974bec39a17e36ba4a6b4d238ff944bacb478cbed5efcae784d7bf4f2ff80"
chain_id = MUMBAI
signer = Signer(private_key=private_key, chain_id=chain_id)

creds = ApiCreds(
    api_key="000000000-0000-0000-0000-000000000000",
    api_passphrase="aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa",
    api_secret="AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA=",
)


class TestHeaders(TestCase):
    def test_create_level_1_headers(self):
        # no nonce
        l1_headers = create_level_1_headers(signer)
        self.assertIsNotNone(l1_headers)
        self.assertEqual(l1_headers[POLY_ADDRESS], signer.address())
        self.assertIsNotNone(l1_headers[POLY_SIGNATURE])
        self.assertIsNotNone(l1_headers[POLY_TIMESTAMP])
        self.assertTrue(
            int(l1_headers[POLY_TIMESTAMP]) <= int(datetime.now().timestamp())
        )
        self.assertEqual(l1_headers[POLY_NONCE], "0")

        # nonce
        l1_headers = create_level_1_headers(signer, nonce=1012)
        self.assertIsNotNone(l1_headers)
        self.assertEqual(l1_headers[POLY_ADDRESS], signer.address())
        self.assertIsNotNone(l1_headers[POLY_SIGNATURE])
        self.assertIsNotNone(l1_headers[POLY_TIMESTAMP])
        self.assertTrue(
            int(l1_headers[POLY_TIMESTAMP]) <= int(datetime.now().timestamp())
        )
        self.assertEqual(l1_headers[POLY_NONCE], "1012")

    def test_create_level_2_headers(self):
        # no body
        l2_headers = create_level_2_headers(
            signer, creds, request_args=RequestArgs(method="get", request_path="/order")
        )
        self.assertIsNotNone(l2_headers)
        self.assertEqual(l2_headers[POLY_ADDRESS], signer.address())
        self.assertIsNotNone(l2_headers[POLY_SIGNATURE])
        self.assertIsNotNone(l2_headers[POLY_TIMESTAMP])
        self.assertTrue(
            int(l2_headers[POLY_TIMESTAMP]) <= int(datetime.now().timestamp())
        )
        self.assertEqual(l2_headers[POLY_API_KEY], creds.api_key)
        self.assertEqual(l2_headers[POLY_PASSPHRASE], creds.api_passphrase)

        # body
        l2_headers = create_level_2_headers(
            signer,
            creds,
            request_args=RequestArgs(
                method="get", request_path="/order", body='{"hash": "0x123"}'
            ),
        )
        self.assertIsNotNone(l2_headers)
        self.assertEqual(l2_headers[POLY_ADDRESS], signer.address())
        self.assertIsNotNone(l2_headers[POLY_SIGNATURE])
        self.assertIsNotNone(l2_headers[POLY_TIMESTAMP])
        self.assertTrue(
            int(l2_headers[POLY_TIMESTAMP]) <= int(datetime.now().timestamp())
        )
        self.assertEqual(l2_headers[POLY_API_KEY], creds.api_key)
        self.assertEqual(l2_headers[POLY_PASSPHRASE], creds.api_passphrase)



import setuptools

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name="py_clob_client",
    version="0.1.0",
    author="Polymarket Engineering",
    author_email="engineering@polymarket.com",
    maintainer="Polymarket Engineering",
    maintainer_email="engineering@polymarket.com",
    description="Python client for the Polymarket CLOB",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/Polymarket/py-clob-client",
    install_requires=["python-dotenv", "py-order-utils>=0.1.0"],
    project_urls={
        "Bug Tracker": "https://github.com/Polymarket/py-clob-client/issues",
    },
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    packages=setuptools.find_packages(),
    python_requires=">=3.9.10",
)



import logging

from .order_builder.builder import OrderBuilder

from .headers.headers import create_level_1_headers, create_level_2_headers
from .signer import Signer

from .endpoints import (
    CANCEL,
    CANCEL_ALL,
    CREATE_API_KEY,
    DELETE_API_KEY,
    DERIVE_API_KEY,
    GET_API_KEYS,
    GET_LAST_TRADE_PRICE,
    GET_ORDER,
    GET_ORDER_BOOK,
    LARGE_ORDERS,
    MID_POINT,
    OPEN_ORDERS,
    ORDER_HISTORY,
    POST_ORDER,
    PRICE,
    TIME,
    TRADE_HISTORY,
)
from .clob_types import (
    ApiCreds,
    FilterParams,
    OrderArgs,
    RequestArgs,
)
from .exceptions import PolyException
from .http_helpers.helpers import add_query_params, delete, get, post
from py_order_utils.config import get_contract_config
from .constants import L0, L1, L1_AUTH_UNAVAILABLE, L2, L2_AUTH_UNAVAILABLE


class ClobClient:
    def __init__(
        self,
        host,
        chain_id: int = None,
        key: str = None,
        creds: ApiCreds = None,
        signature_type: int = None,
        funder: str = None,
    ):
        """
        Initializes the clob client
        The client can be started in 3 modes:
        1) Level 0: Requires only the clob host url
                    Allows access to open CLOB endpoints

        2) Level 1: Requires the host, chain_id and a private key.
                    Allows access to L1 authenticated endpoints + all unauthenticated endpoints

        3) Level 2: Requires the host, chain_id, a private key, and Credentials.
                    Allows access to all endpoints
        """
        self.host = host[0:-1] if host.endswith("/") else host
        self.signer = Signer(key, chain_id) if key else None
        self.creds = creds
        self.mode = self._get_client_mode()
        if chain_id:
            self.contract_config = get_contract_config(chain_id)
        if self.signer:
            self.builder = OrderBuilder(
                self.signer, sig_type=signature_type, funder=funder
            )
        self.logger = logging.getLogger(self.__class__.__name__)

    def get_address(self):
        """
        Returns the public address of the signer
        """
        return self.signer.address() if self.signer else None

    def get_collateral_address(self):
        """
        Returns the collateral token address
        """
        if self.contract_config:
            return self.contract_config.get_collateral()

    def get_conditional_address(self):
        """
        Returns the conditional token address
        """
        if self.contract_config:
            return self.contract_config.get_conditional()

    def get_exchange_address(self):
        """
        Returns the exchange address
        """
        if self.contract_config:
            return self.contract_config.get_exchange()

    def get_executor_address(self):
        """
        Returns the executor address
        """
        if self.contract_config:
            return self.contract_config.get_executor()

    def get_ok(self):
        """
        Health check: Confirms that the server is up
        Does not need authentication
        """
        return get("{}/".format(self.host))

    def get_server_time(self):
        """
        Returns the current timestamp on the server
        Does not need authentication
        """
        return get("{}{}".format(self.host, TIME))

    def create_api_key(self, nonce: int = None):
        """
        Creates a new CLOB API key for the given
        """
        self.assert_level_1_auth()

        endpoint = "{}{}".format(self.host, CREATE_API_KEY)
        headers = create_level_1_headers(self.signer, nonce)

        creds = post(endpoint, headers=headers)
        self.logger.info(creds)
        return creds

    def derive_api_key(self, nonce: int = None):
        """
        Derives an already existing CLOB API key for the given address and nonce
        """
        self.assert_level_1_auth()

        endpoint = "{}{}".format(self.host, DERIVE_API_KEY)
        headers = create_level_1_headers(self.signer, nonce)

        creds = get(endpoint, headers=headers)
        self.logger.info(creds)
        return creds

    def get_api_keys(self):
        """
        Gets the available API keys for this address
        Level 2 Auth required
        """
        self.assert_level_2_auth()

        request_args = RequestArgs(method="GET", request_path=GET_API_KEYS)
        headers = create_level_2_headers(self.signer, self.creds, request_args)
        return get("{}{}".format(self.host, GET_API_KEYS), headers=headers)

    def delete_api_key(self):
        """
        Deletes an API key
        Level 2 Auth required
        """
        self.assert_level_2_auth()

        request_args = RequestArgs(method="DELETE", request_path=DELETE_API_KEY)
        headers = create_level_2_headers(self.signer, self.creds, request_args)
        return delete("{}{}".format(self.host, DELETE_API_KEY), headers=headers)

    def get_midpoint(self, tokenID):
        """
        Get the mid market price for the given market
        """
        return get("{}{}?market={}".format(self.host, MID_POINT, tokenID))

    def get_price(self, tokenID, side):
        """
        Get the market price for the given market
        """
        return get("{}{}?price={}&side={}".format(self.host, PRICE, tokenID, side))

    def get_large_orders(self, params: FilterParams = None):
        """
        Gets large orders
        """
        url = add_query_params("{}{}".format(self.host, LARGE_ORDERS), params)
        return get(url)

    def create_order(self, order_args: OrderArgs):
        """
        Creates and signs an order
        Level 2 Auth required
        """
        self.assert_level_2_auth()

        return self.builder.create_order(order_args)

    def post_order(self, order):
        """
        Posts the order
        """
        self.assert_level_2_auth()
        body = order.dict()
        headers = create_level_2_headers(
            self.signer,
            self.creds,
            RequestArgs(method="POST", request_path=POST_ORDER, body=body),
        )
        return post("{}{}".format(self.host, POST_ORDER), headers=headers, data=body)

    def create_and_post_order(self, order_args: OrderArgs):
        """
        Utility function to create and publish an order
        """
        ord = self.create_order(order_args)
        return self.post_order(ord)

    def cancel(self, order_id):
        """
        Cancels an order
        Level 2 Auth required
        """
        self.assert_level_2_auth()
        body = {"orderID": order_id}

        request_args = RequestArgs(method="DELETE", request_path=CANCEL, body=body)
        headers = create_level_2_headers(self.signer, self.creds, request_args)
        return delete("{}{}".format(self.host, CANCEL), headers=headers, data=body)

    def cancel_all(self):
        """
        Cancels all available orders for the user
        Level 2 Auth required
        """
        self.assert_level_2_auth()
        request_args = RequestArgs(method="DELETE", request_path=CANCEL_ALL)
        headers = create_level_2_headers(self.signer, self.creds, request_args)
        return delete("{}{}".format(self.host, CANCEL_ALL), headers=headers)

    def get_open_orders(self, params: FilterParams = None):
        """
        Gets open orders for the API key
        Requires Level 2 authentication
        """
        self.assert_level_2_auth()
        request_args = RequestArgs(method="GET", request_path=OPEN_ORDERS)
        headers = create_level_2_headers(self.signer, self.creds, request_args)
        url = add_query_params("{}{}".format(self.host, OPEN_ORDERS), params)
        return get(url, headers=headers)

    def get_order_book(self, token_id):
        """
        Fetches the orderbook for the token_id
        """
        return get("{}{}?market={}".format(self.host, GET_ORDER_BOOK, token_id))

    def get_order(self, order_id):
        """
        Fetches the order corresponding to the order_id
        Requires Level 2 authentication
        """
        self.assert_level_2_auth()
        endpoint = "{}{}".format(GET_ORDER, order_id)
        request_args = RequestArgs(method="GET", request_path=endpoint)
        headers = create_level_2_headers(self.signer, self.creds, request_args)
        return get("{}{}".format(self.host, endpoint), headers=headers)

    def get_trade_history(self, params: FilterParams = None):
        """
        Fetches the trade history for a user
        Requires Level 2 authentication
        """
        self.assert_level_2_auth()
        request_args = RequestArgs(method="GET", request_path=TRADE_HISTORY)
        headers = create_level_2_headers(self.signer, self.creds, request_args)
        url = add_query_params("{}{}".format(self.host, TRADE_HISTORY), params)
        return get(url, headers=headers)

    def get_order_history(self, params: FilterParams = None):
        """
        Fetches order history for a user
        Requires Level 2 Authentication
        """
        self.assert_level_2_auth()
        request_args = RequestArgs(method="GET", request_path=ORDER_HISTORY)
        headers = create_level_2_headers(self.signer, self.creds, request_args)
        url = add_query_params("{}{}".format(self.host, ORDER_HISTORY), params)
        return get(url, headers=headers)

    def get_last_trade_price(self, token_id):
        """
        Fetches the last trade price token_id
        """
        return get("{}{}?market={}".format(self.host, GET_LAST_TRADE_PRICE, token_id))

    def assert_level_1_auth(self):
        """
        Level 1 Poly Auth
        """
        if self.mode < L1:
            raise PolyException(L1_AUTH_UNAVAILABLE)

    def assert_level_2_auth(self):
        """
        Level 2 Poly Auth
        """
        if self.mode < L2:
            raise PolyException(L2_AUTH_UNAVAILABLE)

    def _get_client_mode(self):
        if self.signer is not None and self.creds is not None:
            return L2
        if self.signer is not None:
            return L1
        return L0
